#!/bin/sh
exec $ZEUSHOME/perl/miniperl -x $0 ${1+"$@"}
 if 0;
#!/usr/bin/perl -w

BEGIN { @INC=( "$ENV{ZEUSHOME}/perl/", "$ENV{ZEUSHOME}/webadmin/etc/" ); }
use ZeusOS;
use WebAdmin;

# First off, ensure this page isn't cached, otherwise it can be very
# confusing!
print "Pragma: no-cache\n";
print "Cache-Control: no-cache\n";
print "Expires: 0\n";

# Determine if we are Zeus Lite
$lite = 1 if -e "$ENV{ZEUSHOME}/webadmin/LITE";

# Find out how many of the webservers are alive
#
# $webserver_alive defined if any webserver processes are alive
# $webserver_local defined if there is 1 webserver, and its local
#
# Number of webservers installed
$webserver_installed = 0;

# Indentation size
$indent_width = 18;

# If we have this number of websites or less, use just the original display
$min_sites = 10;

# The images used
$image[ 0 ] = "<img border=0 src=\"closed.gif\" alt=\">\">";
$image[ 1 ] = "<img border=0 src=\"open.gif\" alt=\"V\">";
$image[ 2 ] = "";

# Max number of 'forks' at each depth in the tree
$maxforks = 8;
 
# A 'leaf' should contain about this many websites
$leafsize = $min_sites;

$confdir = "$ENV{ZEUSHOME}/webadmin/conf";
$hostsfile = "$confdir/hosts";
$lbsfile = "$confdir/lbs";
$cookiename = "ZEUSA";

# Process the query parameters, given by the URL or cookies
get_parameters();

# Read in the list of sites, runningsites and deadsites
get_sites();


#############################################################################
# Start printing out the page
print "Set-Cookie: ZEUSAPPROOT=/apps/web; path=/apps/\n";
print "Content-Type: text/html\n\n";
printf( "<html><head><title>%s</title></head>",
        $lite ? "Zeus Lite Controller" : "Zeus Web Server Controller" );

print qq{ <body bgcolor="#ffffff" text="#000000" link="#990066"
          vlink="#990066" alink="#990066">
          <table border=0>
          <tr>
          <td valign=top>
          <img src="title.gif" hspace=10 alt="">
          <p align=center>
        };

# Provide a box to search through the websites
if( $original_sites > $min_sites ) {
   print qq{ <form method="get" action="index.cgi">Search servers: };
   if( $filter ne "" ) {
      print "<a href=\"index.cgi?&tree=&show=$show\">" .
        "[all]</a>";
   }
   print qq{ <br>
             <input type="text" name="filter" value="$filter" size=15>
             <input type="image" name="[go]" src="search.gif"
             align="absmiddle" border=0 hspace=0 vspace=0>
             </form>
           };
}

print qq{ <p align=right>
          <a href="/apps/admin/docsystem/context.ahtml?help=/apps/web/docs/controller/controller.html" target=help onClick="window.open('','help','resizable=1,width=640,height=500'); return true;">
          <img src="/apps/web/docs.gif" 
          alt="[documentation]" align=bottom border=0 hspace=0></a>
          <a href="/"><img src="back.gif" alt="[back]" align=bottom border=0 hspace=0></a>
          </td>
          <td>
          <img src="status.gif" alt="Status"><p>
        };

# Depending on whether the webserver is running or not, provide
# a link to start or stop the webserver process.
# Note this is different in the clustering case

if( $webserver_installed == 0 ) {
   print "<i>No webserving machines in the server farm are configured." .
         "<a href=cluster/settings.ahtml>Register machine</a>.</i>\n";
} else {
   if( $webserver_alive ) {
      if( $webserver_local ) {
         print "Webserver running (<a href=\"http.cgi?stop\">Stop</a>)";
      } else {
         print "$webserver_alive/$webserver_installed Webserver";
         print "s" if ( $webserver_installed > 1 );
         print " running";
      }
   } else {
      if( $webserver_local ) {
         print "Webserver not running (<a href=\"http.cgi?start\">Start</a>)";
      } else {
         print "<b>No machines in server farm running</b>";
      }
   }
}
print "<p>\n";
if( $frontends_installed > 0 ) {
   print "$frontends_alive/$frontends_installed Zeus Load Balancer";
   print "s" if( $frontends_installed > 1 );
   print " running<p>\n";
}

# Filtering controls
if( $filter ne "" ) {
   print "Virtual servers matching the regular expression '$filter' are " .
         "shown below:";
} else {
   if( $filter_error ne "" ) {
      print "The regular expression '$filter_bad' is invalid: $filter_error";
   } else {
      if( @sites > $min_sites ) {
         print "All virtual servers are shown below:";
      }
   }
}

print "<p>";

if( @sites == 0 ) {
   if( $original_sites > 0 ) {
      print "<i>No servers matched the given pattern.</i>";
   } else {
      print "<i>No servers available to configure, please " .
            "<a href=new.ahtml>create one</a>.</i>";
   }
} else {
   # If we are showing all websites, and there are going to be a lot of them,
   # don't bother with tables, just print out PRE formatted text. This will
   # make the whole page render _much_ faster
   $table = 1;

   if( $show eq "all" ) {
      if   ( $tree eq "0-1" ) { $num = @deadsites; }
      elsif( $tree eq "1-1" ) { $num = @runningsites; }
      else { $num = @sites; };
      if( $num > 100 ) { $table = 0; };
   }

   if( $table ) {
      print "<center><table border=0 cellspacing=1 cellpadding=1 " .
            "width=\"100%\">\n";
   } else {
      print "<PRE>";
   }

   # If we have 10 websites or fewer, then don't bother with all this fancy
   # tree layout; use the original way of drawing the traffic lights!
   if( @sites <= $min_sites ) {
      $show = "all";
      $tree = "2-1";
      $view = "";
      @tree = ( "2-1" );
      expand_tree( \@sites, "2", 0, -1 );
   } else {
      # The tree has three toplevel branches
      # -+->Sites not running
      #  +->Sites running
      #  +->Both
      
      # The tree is stored as 'depth-name' pairs. The depth is a string of
      # numbers showing the path taken to reach this branch. The three
      # toplevel branches are 0-2 respectively. '0421-attender' means that
      # in the not-running section, taking branch 4, then 2, then 1, we have
      # a subsection open which starts with the website 'attender'.
      # The tree data is listed in no particular order!

      if( $view ) { set_view() };

      $toggle = is_open( "2", \@tree );
      new_heading( "All virtual servers", $#sites, $toggle ? 1 : 0, "2" );
      if ( $toggle ) {
         expand_tree( \@sites, "2", 0, -1 );
      }

      $toggle = is_open( "0", \@tree );
      new_heading( "Virtual servers not running", $#deadsites,
                   $toggle ? 1 : 0, "0" );
      if ( $toggle ) {
         expand_tree( \@deadsites, "0", 0, -1 );
      }
      
      $toggle = is_open( "1", \@tree );
      new_heading( "Virtual servers running",
                   $#runningsites, $toggle ? 1 : 0, "1" );
      if ( $toggle ) {
         expand_tree( \@runningsites, "1", 0, -1 );
      }
   }
   if( $table ) { print "</table></center>"; };
}

# Print the end of the page
print qq{
         <p><br><img src="actions.gif" alt="Actions"><p>
         <center>
         <table cellspacing=10>
         <tr>
         <td align=center>
         <a href=new.ahtml><img src="button-new.gif" alt="*" border=0></a>
         </td>
        };
   
if( !$lite ) {
   print qq{
            <td align=center>
            <a href="cluster/status.ahtml"><img src="cluster/button-cluster.gif" alt="*" border=0></a>
            </td>
            <td align=center>
            <a href="graph.cgi?back=/apps/web/"><img src="button-graph.gif" alt="*" border=0></a>
            </td>
           };
}
   
print qq{
         </tr>
         <tr>
         <td align=center valign=top>
         <a href=new.ahtml>New server</a><br>
         </td>
        };
   
if( !$lite ) {
   print qq{
            <td align=center valign=top>
            <a href="cluster/status.ahtml">Clustering</a><br>
            </td>
            <td align=center valign=top>
            <a href="graph.cgi?back=/apps/web/">Activity<br>monitor</a>
            </td>
           };
}
print "</tr></table></center></td></tr></table></body></html>\n";
exit( 0 );



############################################################################

# Get the page preferences
sub get_parameters()
{
   $cookiename = "ZEUSA";

   # Draw some traffic lights. But which ones? Check the parameters given
   # in order to work out the tree.
   if( $ENV{ QUERY_STRING } && ( $ENV{ QUERY_STRING } ne "" )) {
      $query = $ENV{ QUERY_STRING };
      
      # Set a cookie, so that if a user is directed back to this page, they
      # inherit the same options, so the tree of websites stays opened
      # (hmm, lynx ignores us re-setting a cookie. Clearing the cookie then
      # setting it again makes lynx prompt about a cookie each time you view
      # a section of the page, but its still not working properly.)
      print "Set-Cookie: $cookiename=$query\n";
      
   } elsif( $ENV{ HTTP_COOKIE } && ( $ENV{ HTTP_COOKIE } ne "" )) {
      # No information given. Hopefully, this client has given us a cookie
      # containing their previous options...
      ( $query ) = ( $ENV{ HTTP_COOKIE } ) =~ /$cookiename=(.*?)($|;)/;
      if( !$query ) { $query = "" };
   } else {
      $query = "";
   }
   
   $_ = "\&" . ( $query ) . "\&";
   
   # History of where we are in the tree
   ( $tree ) = /&tree=(.*?)&/;
   $tree ||= "";
   # Show all servers?
   ( $show ) = /&show=(.*?)&/;
   $show ||= "";
   # Filter servers?
   ( $filter ) = /&filter=(.*?)&/;
   $filter ||= "";
   # View a specific website?
   ( $view ) = /&view=(.*?)&/;

   # Generate the default options to be used in links from this page
   $opts = "";
   if( $filter ne "" ) {
      { $opts .= "&filter=$filter"; };

      # Decode the filter if it has been mangled in the URL
      while( ( $hex ) = $filter =~ /%(\w{2})/ ) {
         $char = chr( hex( $hex ));
         $filter =~ s/%$hex/$char/g;
      }
   }

   # Check that the regular expression is valid
   eval "/$filter/";
   $filter_error = $@;
   if( $@ ne "" ) {
      $_ = $@;
      ( $filter_error ) = /(.*)\ at/;
      $filter_bad = $filter;
      $filter = "";
   };

   # Decode the tree into depth-name pairs
   @tree = split( ',', $tree );
}


sub get_sites()
{
   if( open( HOSTS, "<$hostsfile" )) {
      while( <HOSTS> ) {
         next unless( /:/ );
         chomp;
         ( $host, $port ) = split ':';
         $webserver_local = 1 if( $host eq "localhost" );
         $webserver_installed ++;
         
         if( $port != 0 ) {
            # Try and contact the server
            my $error = "";
            my $result = WebAdmin::command( $host, $port, "info",
                                            $error, 2 );
            $webserver_alive ++ if $result;
         } else {
            # This is a non-Zeus webserver, we have no way to tell if its
            # alive or not
            $webserver_alive ++;
         }
      }
      close( HOSTS );
      $webserver_local = 0 if( $webserver_installed != 1 );
   }

   # If there are only load balancers, report on them too
   $frontends_installed = 0;
   $frontends_alive = 0;
   if( open( LBS, "<$lbsfile" )) {
      while( <LBS> ) {
         next unless( /:/ );
         chomp;
         ( $host, $port ) = split ':';
         $frontends_installed ++;
         my $error = "";
         my $result = WebAdmin::command( $host, $port, "info", $error, 2 );
         $frontends_alive ++ if $result;
      }
      close( LBS );
   }
         
   # Work out what sites we have
   @sites = ();
   if( opendir( SITES, "$confdir/sites" )) {
      @sites = sort {uc($a) cmp uc($b)} ( grep !/^\./, readdir SITES );
      closedir( SITES );
   } else { print "$!"; }
   
   # And also what running sites we have
   @runningsites = ();
   if( opendir( RUNNINGSITES, "$confdir/runningsites" )) {
      @runningsites = sort {uc($a) cmp uc($b)} ( grep !/^\./, readdir RUNNINGSITES );
      closedir( RUNNINGSITES );
   } else { print "$!"; }
   
   $original_sites = @sites;
   
   # If a filter was given, filter both lists through the pattern
   if( $filter ne "" ) {
      @sites = grep /$filter/i, @sites;
      @runningsites = grep /$filter/i, @runningsites;
   }
   
   # From the two lists, derive a list of non-running sites
   @deadsites = @sites;
   $deadptr = 0;
   $liveptr = 0;
   while(( $deadptr < @deadsites ) && ( $liveptr < @runningsites )) {
      $compare = lc($deadsites[ $deadptr ]) cmp lc($runningsites[ $liveptr ]);
      if( $compare == -1 ) { $deadptr ++; }
      elsif( $compare == 1 ) { $liveptr ++; }
      else { splice( @deadsites, $deadptr, 1 ); }
   }
}


# Check whether the given branch is open. If it is, return the website name
# of the branch.
sub is_open( $ )
{
   my $branch = shift;
   my $i;
   
   for( $i = 0; $i < @tree; $i ++ ) {
      if( $tree[ $i ] =~ /^$branch-/ ) {
         my $name;
         ( $_, $name ) = split( '-', $tree[ $i ], 2 );
         return $name;
      }
   }
   return;
}


# Find a given website in a list of sites. Returns the position in the list
# if found, or -1 if its not there.
sub find_site( @$ )
{
   my( $sites, $find ) = @_;
   my $s = 0;
   my $e = @$sites - 1;
   return -1 unless $e >= 0;
   my( $pos, $cmp );
   do {
      $pos = int(( $s + $e ) / 2 );
      $cmp = lc($$sites[ $pos ]) cmp lc($find);
      if( $cmp == 1 ) { $e = $pos - 1; }
      elsif( $cmp == -1 ) { $s = $pos + 1; };
   } while(( $cmp != 0 ) && ( $s <= $e ));

   if( $cmp == 0 ) { return $pos; } else { return -1 };
}



# Remove a branch from the tree. Returns the modifed tree (in string form).
sub remove_from_tree( $ )
{
   my $branch = shift;
   my $i;
   my $t = "";
   for( $i = 0; $i < @tree; $i ++ ) {
      if( $tree[ $i ] !~ /^$branch-/ ) {
         $t .= "$tree[ $i ],";
      }
   }
   chop $t;
   return $t;
}


# Add a branch from the tree. Returns the modified tree (in string form).
sub add_to_tree( $ )
{
   my $branch = shift;
   my $i;
   for( $i = 0; $i < @tree; $i ++ ) {
      if( $tree[ $i ] eq $branch ) { return; };
   }

   my $t = $tree;
   if( $t ne "" ) { $t .= ","; };
   $t .= $branch;
   return $t;
}


# If a 'view' parameter was given, then ensure that the correct branches
# are opened to let this site be seen
sub set_view()
{
   $pos = find_site( \@deadsites, $view );
   if( $pos >= 0 ) {
      # Found in deadsites, so open deadsites branch
      $viewsection = '0';
      $sites = \@deadsites;
   } else {
      # Well, must be in runningsites then
      $viewsection = '1';
      $sites = \@runningsites;
   }
   $tree = add_to_tree( "$viewsection-1" );

   # Scan through the sites list, opening sub-branches
   set_sub_view( $sites, "$viewsection", 0, $#$sites );
   @tree = split( ',', $tree );
}


# Kind of like expand_tree(), 'cept we don't draw anything here, we just
# traverse the list of websites & work out which branches to open to show
# site $view
sub set_sub_view( \@$$$ )
{
   my( $sites, $branch, $start, $end ) = @_;
   my( $name, $i );
   my $number = $end - $start;
   my $depth = length( $branch );
   my $forks = int( $number / $leafsize ) + 1;
   if( $forks > $maxforks ) { $forks = $maxforks };
   if( $forks > 1 ) {
      for( $i = 0; $i < $forks; $i ++ ) {
         my $a = int( $number * $i / $forks ) + $start;
         my $b = int( $number * ( $i + 1 ) / $forks ) + $start - 1;
         if( $i == $forks - 1 ) { $b = $end; };
         
         if(( $view ge $$sites[ $a ] ) && ( $view le $$sites[ $b ] )) {
            my $nextbranch = "$branch" . $i;
            $tree = add_to_tree( "$nextbranch-$$sites[ $a ]" );
            set_sub_view( $sites, $nextbranch, $a, $b );
         }
      }
   }
}


# Draw a new heading
sub new_heading( $$$$ )
{
   my( $title, $number, $type, $section ) = @_;
   my $href = "index.cgi?$opts&tree=";
   if( $type == 0 ) {
      $href .= add_to_tree( "$section-1" );
   } else {
      $href .= remove_from_tree( $section );
   }
   $number ++;

   new_indent( 0, 2, $type, $href );
   if( $table ) {
      print "<a href=\"$href\">$title: $number.</a></td><td align=right>" .
            "<a href=\"index.cgi?$opts&tree=$section-1&show=all\">[all]" .
            "</a></td></tr>\n";
   } else {
      print "<a href=\"$href\">$title: $number.</a>" .
            "<a href=\"index.cgi?$opts&tree=$section-1&show=all\">[all]" .
            "</a>\n";
   }
}


# Draw an indent, starting a new table row
sub new_indent( $$$$ )
{
   my( $indent, $span, $type, $link ) = @_;
   if( $table ) {
      print "<tr><td colspan=$span>";
      if( $indent != 0 ) {
         print "<img src=\"single-white.gif\" height=1 width=" .
               ( $indent_width * $indent ) . " alt=\"" .
               ( "&nbsp;" x $indent ) . "\">";
      }
   }
   if( $link ) { print "<a href=\"$link\">"; }
   if( $table) { print $image[ $type ] };
   if( $link ) { print "</a>"; }
}


# Draw parts of a tree.
# $sites is the list of sites from which this part of the tree is constructed.
# $start & $end are the section of the list which we're interested in.
# $branch is the branch history to reach this fork.
sub expand_tree( \@$$$ )
{
   my( $sites, $branch, $start, $end ) = @_;
   my( $name, $i );
   if( $end == -1 ) {
      $end = @$sites - 1;
      if( $end == -1 ) { return; }
   }        
   my $number = $end - $start;
   my $depth = length( $branch );
   my $forks = int( $number / $leafsize ) + 1;
   if( $forks > $maxforks ) { $forks = $maxforks };

   # Are we at a leaf node?
   if(( $show eq "all" ) || ( $forks < 2 )) {
      draw_lights( $sites, $start, $end, $branch );
      return;
   }

   # Draw the next level on the tree
   for( $i = 0; $i < $forks; $i ++ ) {

      my $a = int( $number * $i / $forks ) + $start;
      my $b = int( $number * ( $i + 1 ) / $forks ) + $start - 1;
      if( $i == $forks - 1 ) { $b = $end; };

      # Does this branch open up?
      my $nextbranch = "$branch" . $i;
      my $isopen = is_open( $nextbranch );
      my $t;

      if( $isopen ) {
         # This branch is open, so check that our calculated position 
         # actually is the website specified in the tree (otherwise the
         # tree will become out-of-sync if someone else adds/removes a server
         # whilst we navigate the tree.
         if( $$sites[ $a ] ne $isopen ) {
            my $pos = find_site( $sites, $isopen );
            if( $pos >= 0 ) { $a = $pos };
         }
         # Clicking this link will close this branch - throw away the
         # corresponding element of the tree.
         $t = remove_from_tree( $nextbranch );
      } else {
         $t = add_to_tree ( "$nextbranch-$$sites[ $a ]" );
      }
      my $href = "index.cgi?$opts&tree=$t";
      new_indent( $depth, 3, $isopen ? 1 : 0, $href );
      print "<a href=\"$href\">" .
            "$$sites[ $a ] ... $$sites[ $b ]</a>";
      if( $table ) { print "</td></tr>\n"; };
      
      # Is this branch to be expanded?
      if( $isopen ) {
         expand_tree( $sites, $nextbranch, $a, $b );
      }
   }
}

# Draw a leaf of the tree - traffic lights
sub draw_lights( $$$$ )
{
   my( $sites, $start, $end, $branch ) = @_;
   my $depth = length( $branch );

   if( $table ) {
      print "<tr><td colspan=4><table width=\"100%\" " .
            "cellspacing=0 cellpadding=0 border=0>" .
            "<tr><td width=" . ( $depth * $indent_width ) .
            ">&nbsp;</td><td><table cellspacing=1 cellpadding=5 border=1>\n";
   }

   for( $i = $start; $i <= $end; $i ++ ) {
      my $site = $$sites[ $i ];
      my( $running, $action );
      
      # Find out information about this website
      my( $port, $secure, $ipname ) = read_website_config( $site );
      if( defined $ipname ) {
         if( $secure ) {
            $name = "https://$ipname";
            if( $port != 443 ) { $name .= ":$port"; }
         } else {
            $name = "http://$ipname";
            if( $port != 80 ) { $name .= ":$port"; }
         }
         
         # If this branch is in the 'all sites' list, then we have to
         # determine whether or not each website is running
         if( $branch =~ /^2/ ) {
            if( find_site( \@runningsites, $site) >= 0 ) { $running = 1; };
         } elsif( $branch =~ /^1/ ) { $running = 1; }
         
         if( $webserver_alive ) {
            if( $running ) {
               $alt = "running";
               $graphic = "green.gif";
               $action = "website.ahtml?website=$site&action=stop";
            } else {
               $alt = "stopped";
               $graphic = "red.gif";
               $action = "website.ahtml?website=$site&action=start";
            }
            # Unless we're in the 'all sites' section, starting or stopping
            # a site must change the view so that the site stays visible
            if( $branch !~ /^2/ ) { $action .= "&view="; };               
         } else {
            if( $running ) {
               $alt = "was running";
               $graphic = "green_grayed.gif";
            } else {
               $alt = "was stopped";
               $graphic = "red_grayed.gif";
            }
         }
      } else {
         if( $port ) {
            $name = "Website $site not completely configured";
         } else {
            $name = "Couldn't read config for website $site";
         }
         $graphic = "red_grayed.gif";
         $alt = "error";
      }

      my $modified = ( $running && ( -M "$confdir/runningsites/$site" >
                                     -M "$confdir/sites/$site" ));
      
      # Finally, draw the site table
      if( $table ) {
         printf( "<tr><td>%s<img border = 0 align=top src=$graphic " .
                 "alt=\"[$alt]\">%s</td><td%s>$site %s<br><i>%s" .
                 "</i></td><td><a href=\"edit.ahtml?server=$site\">" .
                 "<img src=edit.gif border=0 alt=\"[edit]\"></a></td></tr>\n",
                 $action ? "<a href=\"$action\">" : "",
                 $action ? "</a>" : "",
                 ( $view && ( $view eq $site )) ? " bgcolor=\"e0e0e0\"" : "",
                 $modified ? "<i>(modified)</i>" : "",
                 ( $webserver_alive && $running )
                   ? "<a href=\"$name\" target=\"running website\">$name</a>"
                   : "<font color=\"#808080\">$name</font>" );
      } else {
         printf( "%s%-12s%s %-20s <i>%s%-40s%s</i> %s\n",
                 $action ? "<a href=\"$action\">" : "",
                 $alt,
                 $action ? "</a>" : "",
                 $modified ? "$site (modified)" : $site,
                 ( $webserver_alive && $running )
                 ? "<a href=\"$name\" target=\"running website\">"
                 : "<font color=\"#808080\">",
                 $name,
                 ( $webserver_alive && $running ) ? "</a>" : "</font>",
                 "<a href=\"edit.ahtml?server=$site\">[edit]</a>" );
      }
   }

   if( $table ) { print "</table></td></tr></table></td></tr>\n"; };
}


# Read the configuration of a particular website. This will return a list
# containing the site's port number, whether its secure or not, and the site's
# hostname.
# If for some reason we can't read the config, the list contents will be
# unset.
sub read_website_config( $ )
{
   my( $site ) = @_;
   my( $port, $secure, $ipname );
   if( open( SITE, "<$confdir/sites/$site" ) ) {
      while( <SITE> ) {
         chomp;
         ( $port ) = /^port\ (.*)/ unless $port;
         ( $secure ) = /^security!enabled\ (yes)/ unless $secure;
         ( $ipname ) = /^ip_name\ (.*)/ unless $ipname;
      }
      close( SITE );
   }
   return( $port, $secure, $ipname );
}
