#!/bin/sh
exec $ZEUSHOME/perl/miniperl -x $0 ${1+"$@"}
 if 0;
#!perl -w

BEGIN { @INC=( "$ENV{ZEUSHOME}/perl/", "$ENV{ZEUSHOME}/webadmin/etc/" ); }
use ZeusOS;
use WebAdmin;

# Args:
#
# machine = name
# vserver = name|* <optional>
# value   = bytes|hits
#
# Plots and refreshes graph of:
#   top $n vservers (if !defined $arg{'vserver'})
# or
#   top $n hheaders for specified vserver
#
# Output Content-type=image/gif

$n = 12;

&parse_form_data(*arg);

# Sanify results
$machine = $arg{'machine'};
$vserver = (defined $arg{'vserver'})?$arg{'vserver'}:"*";
$vserver = "*" if ($vserver eq "_sum");
$value   = (defined $arg{'value'} && $arg{'value'} eq "bytes")?"bytes":"hits";

my ($scale) = ($value eq "bytes")?60:1;

my ($pid) = statd_query_machine($machine, "D", 
   "$value\t$vserver\t\t$n\t199901010000 1 $scale");
   
Error ("Could not fork query") if ($pid<=0);

while (wait() != $pid) {};

if (!open IN, "/tmp/statd.$pid.dat") { 
   Error ("Statd query did not complete");
}

my ($temp, $data);
read IN, $data, (-s "/tmp/statd.$pid.dat");
close IN;
unlink "/tmp/statd.$pid.dat";

my ($stat, $rest) = split "\n", $data, 2;
Error ($stat) if ($stat ne "OK");

($temp, $data) = split /\n/, $rest, 2;
my ($nvals, $time) = split /\t/, $temp;

if ($vserver ne "*") {
   $title = "$machine; Virtual Server: $vserver";
} else {
   # remove the vserver name's as this makes the legend too big
   # int\thostname\tvserver (a%,b) -> int\thostname (a%,b)
   $data =~ s/^(\d+\t\S+)\t\S+/$1/mg;
   $title = "$machine; Busiest Websites";
}

if ($value eq "bytes") {
   $ytitle = "Bytes per second, averaged over $time minutes. ".localtime;
} else {
   $ytitle = "Hits per minute, averaged over $time minutes. ".localtime;
}

open GRAPHER, "| $ENV{'ZEUSHOME'}/webadmin/bin/graph.bar > ./cache/snapshot.$$.gif";
print GRAPHER ($nvals+1)."\n";
print GRAPHER "$title\n";
print GRAPHER "$ytitle\n";
print GRAPHER $data;
close GRAPHER;

send_gif("./cache/snapshot.$$.gif", 10);

# Clean up gifs > 0.001 days (~1.5 minutes) old
unlink grep {-M $_>0.001} <./cache/*>;
exit 0;

# Given a time (int), returns a string in 'web format' suitable for
# HTTP headers, e.g. Thu, 01 Jul 1999 09:22:38 GMT
# This is equivalent to: strftime( "%a, %d %b %Y %H:%M:%S GMT", ... )
#
sub webdate {
  my ( $time )  = @_;
  my ( $ctime ) = scalar gmtime( $time );
  # ctime(3) is pretty close, we look like: Thu Jul  1 09:53:38 1999
  $ctime =~ s/ /, /;                        # Add a comma after the first day
  $ctime =~ s/  / 0/;                       # Fixup 1 digit days of the month
  $ctime =~ s/(\w\w\w) (\d\d)/$2 $1/;       # Swap month name and day of month
  $ctime =~ s/(\d\d:\d\d:\d\d) (.*)/$2 $1/; # Swap time and year
  $ctime .= " GMT";                         # Add timezone info
  return $ctime;
}

sub send_gif {
   my ($file, $refresh) = @_;
   
   open IN, $file or die "Could not open temporary file";
   my ($l) = -s $file;
   my ($e) = webdate( time-1 );
   print <<EOM;
Content-type: image/gif
Pragma: no-cache
Cache-Control: no-cache
Expires: $e
Content-length: $l

EOM
   my ($data);
   read IN, $data, $l;
   print $data;
   close IN;
}

sub Error {
   my ($msg) = @_;
   my ($file) = "./cache/error.$$.gif";
   my ($html);
   
   open GRAPHER, "| $ENV{'ZEUSHOME'}/webadmin/bin/graph.err > $file";
   print GRAPHER $msg;
   close GRAPHER;
   
   send_gif($file, -1);
   # Clean up gifs > 0.001 days (~1.5 minutes) old
   unlink grep {-M $_>0.001} <./cache/*>;
   
   exit 0;
}

sub timetogo {
   exit 0;
}

sub statd_query_machine {
   my($machine, $key, $query) = @_;

   my($pid) = fork;
   
   return (defined $pid)?$pid:-1 if ((defined $pid) && ($pid != 0));

   # In child

   ($host, $port) = split /:/, $machine;
   $command = "statd ".statd_encode($key, "statd.cgi.$$\t$query");

   $response = WebAdmin::command( $host, $port, $command, $err );
   if( !defined($response) ) {
       # Command failed, fake the response wiht embedded error
       $response = "statd.cgi.$$\n$err";
   }

   # Decode response
   ($uniqueid, $stat, $data) = split /\n/, $response, 3;

   my ($out);

   if ($uniqueid ne "statd.cgi.$$") {
      $out = "Response did not match query\n";
   } elsif ($stat ne "OK") {
      $out = $stat."\n";
   } else {
      $out = "OK\n".$data;
   }

   open OUT, ">/tmp/statd.$$.dat"; print OUT $out; close OUT;

   exit 0;
}

sub statd_encode {
   my ($key, $msg) = @_;
   
   return sprintf("%s%04x\t%s", $key, length($msg)+length($key)+5, $msg);
}



sub parse_form_data {
   local (*FORM_DATA) = @_;

   local ($request_method, $query_string, $key_value, $key, $value);

   $request_method = $ENV{'REQUEST_METHOD'};

   if ($request_method eq "GET") {
      $query_string = $ENV{'QUERY_STRING'};
   } elsif ($request_method eq "POST") {
      $query_string = "";
      read STDIN, $query_string, $ENV{'CONTENT_LENGTH'};
   }

   foreach $key_value (split /&/, $query_string) {
      ($key, $value) = split /=/, $key_value;
      $value =~ tr/+/ /;
      $value =~ s/%([\dA-Fa-f][\dA-Fa-f])/pack("C", hex($1))/eg;
      if (defined $FORM_DATA{$key}) {
         $FORM_DATA{$key} = join " ", $FORM_DATA{$key}, $value;
      } else {
         $FORM_DATA{$key} = $value;
      }
   }
   return 0;
}              
